#include "testlib.h"
#include <bits/stdc++.h>

using namespace std;
typedef long long ll;
const ll INF = 0x3f3f3f3f3f3f3f3fLL;

struct Vec2
{
	ll x, y;
	
	Vec2() {}
	Vec2(ll nx, ll ny) : x(nx), y(ny) {}

	Vec2 operator + (const Vec2 & b) const { return { x+b.x, y+b.y }; }
	Vec2 operator - (const Vec2 & b) const { return { x-b.x, y-b.y }; }
	Vec2 operator - () const { return { -x, -y }; }
	Vec2 operator * (ll b) const { return { x*b, y*b }; }
	Vec2 operator / (ll b) const { return { x/b, y/b }; }

	Vec2 & operator += (const Vec2 & b) { x+=b.x; y+=b.y; return *this; }
	Vec2 & operator -= (const Vec2 & b) { x-=b.x; y-=b.y; return *this; }
	Vec2 & operator *= (ll b) { x*=b; y*=b; return *this; }
	Vec2 & operator /= (ll b) { x/=b; y/=b; return *this; }

	bool operator == (const Vec2 & b) const { return x==b.x && y==b.y; }
	bool operator != (const Vec2 & b) const { return x!=b.x && y!=b.y; }
	bool operator <  (const Vec2 & b) const { return x==b.x ? y<b.y : x<b.x; }
	bool operator <= (const Vec2 & b) const { return x==b.x ? y<=b.y : x<b.x; }
	bool operator >  (const Vec2 & b) const { return x==b.x ? y>b.y : x>b.x; }
	bool operator >= (const Vec2 & b) const { return x==b.x ? y>=b.y : x>b.x; }

	int half() const
	{
		if (x > 0)
			return 0;
		if (x < 0)
			return 1;
		return y >= 0 ? 0 : 1;
	}
};

ostream & operator << (ostream & out, const Vec2 & v) { return out << v.x << " " << v.y; }
Vec2 operator * (ll b, const Vec2 & a) { return { b*a.x, b*a.y }; }
ll dot(const Vec2 & a, const Vec2 & b) { return a.x*b.x + a.y*b.y; }
ll cross(const Vec2 & a, const Vec2 & b) { return a.x*b.y - a.y*b.x; }
ll length2(const Vec2 & a) { return a.x*a.x + a.y*a.y; }

const int V = 1000;

int main(int argc, char* argv[])
{
    registerGen(argc, argv, 1);
    int n = atoi(argv[1]);
    int longc = (argc > 2 ? atoi(argv[2]) : 0);
	int longv = (argc > 3 ? atoi(argv[3]) : 1000*n / max(1, longc));

	vector < Vec2 > vs;
	for (int i = 0; i < n; ++i)
	{
		if (i < longc)
			vs.emplace_back(rnd.next(-longv, longv), rnd.next(-longv, longv));
		else
			vs.emplace_back(rnd.next(-V, V), rnd.next(-V, V));
		if (vs.back() == Vec2(0, 0))
		{
			vs.pop_back();
			--i;
		}
	}
	
	Vec2 sum = Vec2(0, 0);
	for (Vec2 v : vs)
		sum += v;
	for (Vec2 & v : vs)
	{
		Vec2 dd = Vec2(sum.x / n, sum.y / n);
		v -= dd;
		sum -= dd;
	}
	vs[0] -= sum;

	sort(vs.begin(), vs.end(), [](const Vec2 & a, const Vec2 & b)
	{
		int ah = a.half();
		int bh = b.half();
		return ah == bh ? cross(a, b) > 0 : ah < bh;
	});

	vector < Vec2 > gv;
	for (Vec2 v : vs)
	{
		if (gv.empty() || cross(gv.back(), v) != 0)
			gv.push_back(v);
		else
			gv.back() += v;
	}

	n = (int)gv.size();
	vector < Vec2 > a;
	a.emplace_back(0, 0);
	for (int i = 0; i < n - 1; ++i)
		a.push_back(a.back() + gv[i]);
	Vec2 pmin = Vec2(INF, INF);
	for (Vec2 p : a)
		pmin = min(pmin, p);
	Vec2 pmax = Vec2(-INF, -INF);
	for (Vec2 p : a)
		pmax = max(pmax, p);
	Vec2 add = -pmin - (pmax - pmin) / 2;
	for (Vec2 & p : a)
		p += add;

	cout << n << "\n";
	int st = rnd.next(0, n);
	for (int i = 0; i < n; ++i)
		cout << a[(i + st) % n] << "\n";

	while (true)
	{
		ll ws = 0;
		Vec2 m = Vec2(0, 0);
		int t = rnd.next(0, n - 1);
		ll mw = rnd.next(1, n);
		for (int i = 0; i < n; ++i)
		{
			Vec2 p = a[i];
			ll w = mw * min(abs(t - i), n - abs(t - i)) / (n/2);
			m += p*w;
			ws += w;
		}
		m /= ws;
		for (int dx : {0, -1, 1})
			for (int dy : {0, -1, 1})
			{
				bool ok = true;
				Vec2 m1 = m + Vec2(dx, dy);
				for (int i = 0; i < n; ++i)
					if (cross(a[(i+1)%n] - a[i], m1 - a[i]) <= 0)
					{
						ok = false;
						break;
					}
				if (ok)
				{
					cout << m1 << "\n";
					return 0;
				}
			}
	}
}

