#include <bits/stdc++.h>

const double EPS = 1e-7;

using namespace std;

struct Vec2
{
	double x, y;
	
	Vec2() {}
	Vec2(double nx, double ny) : x(nx), y(ny) {}

	Vec2 operator + (const Vec2 & b) const { return { x+b.x, y+b.y }; }
	Vec2 operator - (const Vec2 & b) const { return { x-b.x, y-b.y }; }
	Vec2 operator - () const { return { -x, -y }; }
	Vec2 operator * (double b) const { return { x*b, y*b }; }
	Vec2 operator / (double b) const { return { x/b, y/b }; }

	Vec2 & operator += (const Vec2 & b) { x+=b.x; y+=b.y; return *this; }
	Vec2 & operator -= (const Vec2 & b) { x-=b.x; y-=b.y; return *this; }
	Vec2 & operator *= (double b) { x*=b; y*=b; return *this; }
	Vec2 & operator /= (double b) { x/=b; y/=b; return *this; }

	bool operator == (const Vec2 & b) const { return x==b.x && y==b.y; }
	bool operator != (const Vec2 & b) const { return x!=b.x && y!=b.y; }
	bool operator <  (const Vec2 & b) const { return x==b.x ? y<b.y : x<b.x; }
	bool operator <= (const Vec2 & b) const { return x==b.x ? y<=b.y : x<b.x; }
	bool operator >  (const Vec2 & b) const { return x==b.x ? y>b.y : x>b.x; }
	bool operator >= (const Vec2 & b) const { return x==b.x ? y>=b.y : x>b.x; }
};

Vec2 operator * (double b, const Vec2 & a) { return { b*a.x, b*a.y }; }
double dot(const Vec2 & a, const Vec2 & b) { return a.x*b.x + a.y*b.y; }
double cross(const Vec2 & a, const Vec2 & b) { return a.x*b.y - a.y*b.x; }
double length2(const Vec2 & a) { return a.x*a.x + a.y*a.y; }
double length(const Vec2 & a) { return sqrt(a.x*a.x + a.y*a.y); }
Vec2 normalize(const Vec2 & a) { return a / sqrt(a.x*a.x + a.y*a.y); }

Vec2 intersect(const Vec2 & p1, const Vec2 & v1, const Vec2 & p2, const Vec2 & v2)
{
	if (abs(cross(normalize(v1), normalize(v2))) < EPS)
		return p1;
	return p1+v1*(cross(p2-p1, v2)/cross(v1, v2));
}

const int N = 100179;

int n;
Vec2 a[N];
Vec2 m;
double ss[N*2];

double solve(int s)
{
	Vec2 v1 = a[s];
	Vec2 v2 = a[(s + 1) % n];
	Vec2 norm = normalize(v2 - v1); norm = Vec2(-norm.y, norm.x);
	Vec2 d1 = v1 - m; d1 = d1 - norm*(2.0*dot(norm, d1));
	Vec2 d2 = v2 - m; d2 = d2 - norm*(2.0*dot(norm, d2));

	double ans = ss[n];

	int fl = 0, fr = n - 1;
	while (fl < fr)
	{
		int f = (fl + fr + 1) / 2;
		Vec2 bv = a[(n + s - f) % n];
		if (cross(d1, bv - v1) < 0)
			fr = f - 1;
		else
			fl = f;
	}
	ans -= ss[s + n] - ss[s + n - fl];
	Vec2 fp = a[(n + s - fl) % n];
	Vec2 fp1 = a[(n + s - fl - 1) % n];
	Vec2 fpm = intersect(fp, fp1 - fp, v1, d1);
	ans -= cross(v1, fpm) + cross(fpm, fp);
	
	fl = 0, fr = n - 1;
	while (fl < fr)
	{
		int f = (fl + fr + 1) / 2;
		Vec2 bv = a[(s + 1 + f) % n];
		if (cross(d2, bv - v2) > 0)
			fr = f - 1;
		else
			fl = f;
	}
	ans -= ss[s + 1 + fl] - ss[s + 1];
	fp = a[(s + 1 + fl) % n];
	fp1 = a[(s + 1 + fl + 1) % n];
	fpm = intersect(fp, fp1 - fp, v2, d2);
	ans -= cross(fpm, v2) + cross(fp, fpm);

	return ans / 2.0;
}

int main()
{
#ifndef LOCAL
	freopen("mirror.in", "r", stdin);
	freopen("mirror.out", "w", stdout);
#endif

	scanf("%d", &n);
	for (int i = 0; i < n; ++i)
		scanf("%lf%lf", &a[i].x, &a[i].y);
	scanf("%lf%lf", &m.x, &m.y);

	ss[0] = 0;
	for (int i = 0; i < n*2; ++i)
		ss[i + 1] = ss[i] + cross(a[i % n], a[(i + 1) % n]);
	
	for (int i = 0; i < n; ++i)
		printf("%.7lf\n", solve(i));

	return 0;
}
