#include "testlib.h"
#include <fstream>
#include <set>
#include <cstdio>
#include <vector>

using namespace std;

vector <int> parent_dsu, rank_dsu;
vector <vector <int> > edges;

struct item
{
    int x, y, size;
    item *l, *r;
    item() { }
    item(int x, int y, int size) : x(x), y(y), size(size), l(NULL), r(NULL) { }
};
typedef item * pitem;

int size(pitem t)
{
	return t ? t->size : 0;
}

void resize(pitem t)
{
	if (t)
	{
		t->size = size(t->l) + size(t->r) + 1;
	}
}

void split(pitem t, int key, pitem &l, pitem &r)
{
    if (!t)
    {
        l = r = NULL;
        return;
    }
    if (key < t->x)
    {
        split(t->l, key, l, t->l), r = t;
    }
    else
    {
        split(t->r, key, t->r, r), l = t;
    }
	resize(t); 
}

void merge(pitem &t, pitem l, pitem r)
{
    if (!l || !r)
    {
        t = l ? l : r;
        return;
    }
    if (l->y > r->y)
    {
        merge(l->r, l->r, r), t = l;
    }
    else
    {
        merge(r->l, l, r->l), t = r;
    }
	return resize(t);
}

void insert(pitem &t, pitem it)
{
    pitem l, r, t2;
    split(t, it->x, l, r);
    merge(t2, l, it);
    merge(t, t2, r);
}

void del(pitem &t, int key)
{
	pitem l, r, tt;
	split(t, key, tt, r);
	split(tt, key, l, tt);
	merge(t, l, r);
}

int search(pitem t, int k)
{
	if (k == size(t->l) + 1)
	{
		return t->x;
	}
	if (size(t->l) > k)
	{
		return search(t->l, k);
	}
	else if (size(t->l) < k)
	{
		k -= size(t->l) + 1;
		return search(t->r, k);
	}
}

int find_set(int v)
{
	if (parent_dsu[v] == v)
	{
		return v;
	}
	return parent_dsu[v] = find_set(parent_dsu[v]);
}

void union_set(int a, int b)
{
	a = find_set(a);
	b = find_set(b);
	if (a != b)
	{
		if (rank_dsu[a] < rank_dsu[b])
		{
			swap(a, b);
		}
		parent_dsu[b] = a;
		if (rank_dsu[a] == rank_dsu[b])
		{
			++rank_dsu[a];
		}
	}
}

int main(int argc, char* argv[])
{
	//freopen("output.txt", "wt", stdout);
    registerGen(argc, argv, 1);

    int n = atoi(argv[1]);

	//tree
    vector<int> p(n);
	parent_dsu.resize(n);
	edges.resize(n);
	rank_dsu.resize(n, 0);
	for (int i = 0; i < n; i++)
	{
		parent_dsu[i] = i;
	}
    for (int i = 1; i < n; i++)
	{
        int ex = rnd.next(0, n - 1);
		if (find_set(ex) == find_set(i))
		{
			--i;
			continue;
		}
		p[i] = ex;
		union_set(ex, i);
	}

    vector<int> perm(n);
    for (int i = 0; i < n; i++)
	{
        perm[i] = i;
	}
    shuffle(perm.begin() + 1, perm.end());

    vector<int> pp(n);
	edges.resize(n);
	set <pair <int, int> > was;
    for (int i = 1; i < n; i++)
	{
        pp[perm[i]] = perm[p[i]];
		edges[perm[p[i]]].push_back(perm[i]);
		edges[perm[i]].push_back(perm[p[i]]);
		was.insert(make_pair(perm[p[i]], perm[i]));
		was.insert(make_pair(perm[i], perm[p[i]]));
	}


	//gamilton's graph
	if (n > 2)
	{
		pitem t = NULL;
		for (int i = 0; i < n; i++)
		{
			if (edges[i].size() <= n / 2)
			{
				int y = rnd.next(1, (int) 1e9);
				pitem it = new item(i, y, 1);
				insert(t, it);
			}
		}
		for (int i = 0; i < n && size(t) > 0; i++)
		{
			bool in = false;
			while (edges[i].size() <= n / 2)
			{
				in = true;
				int pos = rnd.next(1, size(t));
				int v = search(t, pos);
				if (v == i || was.count(make_pair(i, v)))
				{
					continue;
				}
				edges[i].push_back(v);
				edges[v].push_back(i);
				was.insert(make_pair(i, v));
				was.insert(make_pair(v, i));
				if (edges[v].size() > n / 2)
				{
					del(t, v);
				}
			}
			if (in)
			{
				del(t, i);
			}
		}
	}

	//output
	printf("%d\n", n);
	for (int i = 0; i < n; i++)
	{
		printf("%d", edges[i].size());
		if (edges[i].size() > 0)
		{
			printf(" ");
		}
		for (int j = 0; j < edges[i].size(); j++)
		{
			printf("%d", edges[i][j] + 1);
			if (j < edges[i].size() - 1)
			{
				printf(" ");
			}
		}
		printf("\n");
	}

    return 0;
}