import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;

/**
 * @author Ilya Peresadin <pva701@gmail.com>
 * @created 27.09.16
 */

public class Validate {
    public static final int M = 2_00_000;
    public static final int MAX_SUM_LEN = 1_000_000;

    private boolean isLatin(String s) {
        for (int i = 0; i < s.length(); ++i)
            if (!('a' <= s.charAt(i) && s.charAt(i) <= 'z')) {
                return false;
            }
        return true;
    }

    public void run() {
        StrictScanner inf = new StrictScanner(System.in);
        int n = inf.nextInt();
        int m = inf.nextInt();
        inf.nextLine();
        ensureLimits(n, 1, M, "n isn't valid");
        ensureLimits(m, 1, M, "m isn't valid");

        Fenwick f = new Fenwick(n);
        int sumAppend = 0, sumRequests = 0;
        for (int i = 0; i < m; ++i) {
            int tp = inf.nextInt();
            if (tp == 1) {
                int l = inf.nextInt();
                int r = inf.nextInt();
                String s = inf.next();
                inf.nextLine();

                ensure(s.length() != 0, "String in %d query is empty");
                ensureLimits(s.length(), 1, MAX_SUM_LEN, String.format("String is too long in the %d query", i + 1));
                ensure(isLatin(s), "Append string must consist from latin small letters");
                sumAppend += s.length();
                f.add(l - 1, r - 1, s.length());

                ensureLimits(sumAppend, 1, MAX_SUM_LEN, String.format("Total length of strings in append queries more than %d", MAX_SUM_LEN));
                ensure(l >= 1, String.format("Left bound of append query #%d less than 1. l[%d] = %d", i + 1, i + 1, l));
                ensure(r <= n, String.format("Right bound of append query #%d  more than %d. r[%d] = %d", i + 1, n, i + 1, r));
                ensure(l <= r, String.format("Left bound of append query #%d more than right bound. l[%d] = %d, r[%d] = %d", i + 1, i + 1, l, i + 1, r));
            } else if (tp == 2) {
                int id = inf.nextInt();
                int l = inf.nextInt();
                int r = inf.nextInt();
                inf.nextLine();

                ensure(id >= 1, String.format("Index of required string less than 1. i[%d] = %d", i + 1, id));
                ensure(id <= n, String.format("Index of required string more than %d. i[%d] = %d", n, i + 1, id));
                ensure(l <= r, String.format("Left bound of request substring query #%d more than right bound. l[%d] = %d, r[%d] = %d", i + 1, i + 1, l, i + 1, r));

                int len = f.get(id - 1);
                ensure(len >= r, String.format("Length of %d-th string must than r[%d]. r[%d] = %d, but length %d", id, i + 1, i + 1, r, len));

                sumRequests += r - l + 1;
                ensure(sumRequests <= MAX_SUM_LEN, String.format("Sum of request substring queries more than %d", MAX_SUM_LEN));
            } else {
                throw new AssertionError("Unknown type of query");
            }
        }
        inf.close();
    }

    public static class Fenwick {
        private int[] s;
        private int n;

        public Fenwick(int n) {
            this.n = n;
            this.s = new int[n];
        }

        void add(int v, int val) {
            for (; v < n; v |= v + 1) {
                s[v] += val;
            }
        }

        int get(int v) {
            int ret = 0;
            for (; v >= 0; v = (v & (v+1)) - 1) {
                ret += s[v];
            }
            return ret;
        }

        void add(int l, int r, int val) {
            add(l, val);
            add(r + 1, -val);
        }
    }

    public static void main(String[] args) {
        new Validate().run();
    }

    public class StrictScanner {
        private final BufferedReader in;
        private String line = "";
        private int pos;
        private int lineNo;

        public StrictScanner(InputStream source) {
            in = new BufferedReader(new InputStreamReader(source));
            nextLine();
        }

        public void close() {
            ensure(line == null, "Extra data at the end of file");
            try {
                in.close();
            } catch (IOException e) {
                throw new AssertionError("Failed to close with " + e);
            }
        }

        public void nextLine() {
            ensure(line != null, "EOF");
            ensure(pos == line.length(), "Extra characters on line " + lineNo);
            try {
                line = in.readLine();
            } catch (IOException e) {
                throw new AssertionError("Failed to read line with " + e);
            }
            pos = 0;
            lineNo++;
        }

        public String getNextLine() {
            String st = "";
            while (!isEoln()) {
                st = st + next();
            }
            return st;
        }


        public boolean isEoln() {
            return (pos == line.length());
        }

        public String next() {
            ensure(line != null, "EOF");
            ensure(line.length() > 0, "Empty line " + lineNo);
            if (pos == 0)
                ensure(line.charAt(0) > ' ', "Line " + lineNo
                        + " starts with whitespace");
            else {
                ensure(pos < line.length(), "Line " + lineNo + " is over");
                ensure(line.charAt(pos) == ' ', "Wrong whitespace on line "
                        + lineNo);
                pos++;
                ensure(pos < line.length(), "Line " + lineNo + " is over");
                ensure(line.charAt(0) > ' ', "Line " + lineNo
                        + " has double whitespace");
            }
            StringBuilder sb = new StringBuilder();
            while (pos < line.length() && line.charAt(pos) > ' ')
                sb.append(line.charAt(pos++));
            return sb.toString();
        }

        public int nextInt() {
            String s = next();
            ensure(s.length() == 1 || s.charAt(0) != '0',
                    "Extra leading zero in number " + s + " on line " + lineNo);
            ensure(s.charAt(0) != '+', "Extra leading '+' in number " + s
                    + " on line " + lineNo);
            try {
                return Integer.parseInt(s);
            } catch (NumberFormatException e) {
                throw new AssertionError("Malformed number " + s + " on line "
                        + lineNo);
            }
        }

        public long nextLong() {
            String s = next();
            ensure(s.length() == 1 || s.charAt(0) != '0',
                    "Extra leading zero in number " + s + " on line " + lineNo);
            ensure(s.charAt(0) != '+', "Extra leading '+' in number " + s
                    + " on line " + lineNo);
            try {
                return Long.parseLong(s);
            } catch (NumberFormatException e) {
                throw new AssertionError("Malformed number " + s + " on line "
                        + lineNo);
            }
        }

        public double nextDouble() {
            String s = next();
            ensure(s.length() == 1 || s.startsWith("0.") || s.charAt(0) != '0',
                    "Extra leading zero in number " + s + " on line " + lineNo);
            ensure(s.charAt(0) != '+', "Extra leading '+' in number " + s
                    + " on line " + lineNo);
            try {
                return Double.parseDouble(s);
            } catch (NumberFormatException e) {
                throw new AssertionError("Malformed number " + s + " on line "
                        + lineNo);
            }
        }
    }

    static void ensure(boolean b, String message) {
        if (!b) {
            throw new AssertionError(message);
        }
    }

    void ensureString(String s, String alph, String text) {
        for (int i = 0; i < s.length(); i++) {
            ensure(alph.indexOf(s.charAt(i)) != -1, text);
        }
    }

    void ensureLimits(long n, long from, long to, String name) {
        ensure(from <= n && n <= to, name + " must be from " + from + " to "
                + to);
    }

    void ensureLimits(int n, int from, int to, String name) {
        ensure(from <= n && n <= to, name + " must be from " + from + " to "
                + to);
    }

    String line(int x) {
        return "Problem in line #" + Integer.toString(x);
    }
}
