#include <iostream>
#include <vector>
#include <cstdlib>
#include <cstdio>
#include <algorithm>
#include <stdexcept>
using namespace std;
#define TASK "stringsqueries"

const int N = 1e6 + 10;
const int M = 2e5 + 10;

struct Node {
    int x, y, sum;
    const string* val;
    Node *l, *r;
    Node() {}
    Node(int x, const string& val):x(x), val(&val), y(random()), l(NULL), r(NULL) {}

    static Node* nodes;
    static int cnt;
    void* operator new(size_t size) {
        return &nodes[cnt++];
    }

private:
    static int random() {
        return rand() | (rand()<<15);
    }
};

int Node::cnt = 0;
Node* Node::nodes = new Node[M];

typedef Node* PNode;

int sum(PNode v) {
    if (v) {
        return v->sum;
    } else {
        return 0;
    }
}

PNode update(PNode v) {
    if (v) {
        v->sum = sum(v->l) + sum(v->r) + v->val->size();
    }
    return v;
}

PNode merge(PNode L, PNode R) {
    if (!L) return update(R);
    if (!R) return update(L);
    if (L->y > R->y) {
        L->r = merge(L->r, R);
        return update(L);
    } else {
        R->l = merge(L, R->l);
        return update(R);
    }
}

void splitBySum(PNode t, int k, PNode& L, PNode& R) {
    if (!t) L = R = NULL;
    else if (sum(t->l) + t->val->size() <= k) {
        splitBySum(t->r, k - sum(t->l) - t->val->size(), t->r, R);
        L = t;
    } else {
        splitBySum(t->l, k, L, t->l);
        R = t;
    }
    update(t);
}

void split(PNode t, int k, PNode& L, PNode& R) {
    if (!t) L = R = NULL;
    else if (t->x <= k) {
        split(t->r, k, t->r, R);
        L = t;
    } else {
        split(t->l, k, L, t->l);
        R = t;
    }
    update(t);
}

int getSubtreeStrings(PNode root, const string** subtreeStrings) {
    if (!root) {
        return 0;
    }
    int szLeft = getSubtreeStrings(root->l, subtreeStrings);
    subtreeStrings[szLeft] = root->val;
    int szRight = getSubtreeStrings(root->r, subtreeStrings + szLeft + 1);
    return szLeft + szRight + 1;
}

const string& firstString(PNode root) {
    PNode v = root;
    while (v->l != NULL) {
        v = v->l;
    }
    return *v->val;
}

string getSubstring(PNode& t, int i, int j) {
    static const string* subtreeStrings[M];
    PNode L = NULL, M = NULL, R = NULL;
    splitBySum(t, j, L, R);
    int maxLen = (L ? L->sum : 0);
    splitBySum(L, i - 1, L, M);
    string answer;
    if (maxLen < i) {
        const string& oneString = firstString(R);
        i -= sum(L) + 1;
        j -= sum(L) + 1;
        answer = oneString.substr(i, j - i + 1);
    } else {
        int prefLen = sum(M) - (maxLen - i + 1);
        int sizeSubtree = getSubtreeStrings(M, subtreeStrings);
        const string* first = subtreeStrings[0];
        answer = first->substr(prefLen, first->size() - prefLen);
        for (int i = 1; i < sizeSubtree; ++i) {
            answer += *subtreeStrings[i];
        }
        if (maxLen != j) {
            const string& lastString = firstString(R);
            answer += lastString.substr(0, j - maxLen);
        }
    }
    t = merge(merge(L, M), R);
    return answer;
}

void insert(PNode& root, int x, const string& val) {
    PNode L, R, M = new Node(x, val);
    split(root, x, L, R);
    root = merge(merge(L, M), R);
}

void remove(PNode& t, int x) {
    PNode L, M, R;
    split(t, x, L, R);
    split(L, x - 1, L, M);
    t = merge(L, R);
}

char s[N];
struct Event {
    int type, x, i;
    Event() {}
    Event(int type, int x, int i): type(type), x(x), i(i) {}
    bool operator < (const Event& ev) const {
        return x < ev.x || x == ev.x && type < ev.type;
    }
};

Event ev[2*M];
int events;

int main() {
    freopen(TASK".in", "r", stdin);
    freopen(TASK".out", "w", stdout);

    int n, m;
    cin >> n >> m;
    vector<string> strings(m), answers(m);
    vector<int> x(m), y(m);

    for (int i = 0; i < m; ++i) {
        int tp;
        scanf("%d", &tp);
        if (tp == 1) {
            int l, r;
            scanf("%d%d%s", &l, &r, s);
            strings[i] = s;
            ev[events++] = Event(0, l, i);
            ev[events++] = Event(2, r, i);
        } else {
            int p, l, r;
            scanf("%d%d%d", &p, &l, &r);
            x[i] = l;
            y[i] = r;
            ev[events++] = Event(1, p, i);
        }
    }

    PNode root = NULL;
    sort(ev, ev + events);
    for (int i = 0; i < events; ++i) {
        if (ev[i].type == 0) {
            insert(root, ev[i].i, strings[ev[i].i]);
        } else if (ev[i].type == 2) {
            remove(root, ev[i].i);
        } else {
            int id = ev[i].i;
            answers[id] = getSubstring(root, x[id], y[id]);
            if (answers[ev[i].i].size() !=  y[id] - x[id] + 1) {
                throw runtime_error("Substring length incorrect");
            }
        }
    }

    for (int i = 0; i < m; ++i) {
        if (answers[i].size() != 0) {
            printf("%s\n", answers[i].c_str());
        }
    }
    return 0;
}
